@@ -1,5 +1,31 @@
 spg <- function(par, fn, gr=NULL, method=3, project=NULL, 
-           lower=-Inf, upper=Inf,  control=list(), quiet=FALSE,  ... ) {
+           lower=-Inf, upper=Inf, projectArgs=NULL, 
+	   control=list(), quiet=FALSE,  ... ) {
+
+  prj <- TRUE
+  if (is.null(project)){
+     if (is.null(projectArgs)){
+         projectArgs <- list(lower=lower, upper=upper)
+         #  expand upper and lower for all par
+         if (length(projectArgs$lower)==1)
+             projectArgs$lower <- rep(projectArgs$lower, length(par))
+         if (length(projectArgs$upper)==1)
+             projectArgs$upper <- rep(projectArgs$upper, length(par))
+         }
+     if (any(is.finite(c(projectArgs$upper, projectArgs$lower)))){
+         project <- "projectBox"
+         # projectBox for default 
+         # This provides box constraints defined by upper and lower
+         projectBox <- function(par, lower, upper) {
+           # Projecting to ensure that box-constraints are satisfied
+           par[par < lower] <- lower[par < lower]
+           par[par > upper] <- upper[par > upper]
+           return(par)
+           }
+     
+         } else prj <- FALSE    
+     }
+
 
   # control defaults
   ctrl <- list(M=10, maxit=1500, gtol=1.e-05, maxfeval=10000, maximize=FALSE, 
@@ -18,10 +44,7 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
   triter   <- ctrl$triter
   eps      <- ctrl$eps
   checkGrad.tol <- ctrl$checkGrad.tol  
-
-  if (any(is.finite(lower)) & length(lower)==1) lower <- rep(lower, length(par))
-  if (any(is.finite(upper)) & length(upper)==1) upper <- rep(upper, length(par))
-  
+    
   grNULL <- is.null(gr)  
   fargs <- list(...)
   ################ local function
@@ -87,15 +110,6 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
     	df
 	}
 
-
-  # This provides box constraints defined by upper and lower
-  # local functions defined only when user does not specify project.
-  if (is.null(project)) project <- function(par, lower, upper, ...) {
-       # Projecting to ensure that box-constraints are satisfied
-       par[par < lower] <- lower[par < lower]
-       par[par > upper] <- upper[par > upper]
-       return(par)
-       }
   #############################################
 
   #  Initialization
@@ -116,13 +130,13 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
 
 
   # Project initial guess
-  par <- try(project(par, lower, upper, ...), silent=TRUE)
+  if (prj){
+     par <- try(do.call(project,  append(list(par), projectArgs)), silent=TRUE)
  
-  if (class(par) == "try-error") 
-        stop("Failure in projecting initial guess!", par)
-  else if (any(is.nan(par), is.na(par)) ) 
-        stop("Failure in projecting initial guess!")
-  
+     if (class(par) == "try-error") 
+           stop("Failure in projecting initial guess!", par)
+     }
+  if (any(is.nan(par), is.na(par)) ) stop("Failure in initial guess!")
   pbest <- par
  
   f <- try(func(par, ...),silent=TRUE)
@@ -149,12 +163,13 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
  
   lastfv[1] <- fbest <- f
  
-  pg <- try(project(par - g, lower, upper, ...),silent=TRUE)
- 
-  if (class(pg)=="try-error" ) 
-        stop("Failure in initial projection!", pg)
-  else if (any(is.nan(pg))) 
-        stop("Failure in initial projection!")
+  pg <- par - g
+  if (prj){
+    pg <- try(do.call(project,  append(list(pg), projectArgs)),silent=TRUE)
+    if (class(pg)=="try-error" ) stop("Failure in initial projection!", pg)
+    } 
+    
+  if (any(is.nan(pg))) stop("Failure in initial projection!")
  
   pg <- pg - par
 
@@ -163,7 +178,7 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
   gbest <- pg2n
   if (pginfn != 0) lambda <- min(lmax, max(lmin, 1/pginfn))
  
-  if (trace) cat("iter: ",0, " f-value: ", f0, " pgrad: ",pginfn, "\n")
+  if (trace) cat("iter: ",0, " f-value: ", f0 * (-1)^maximize, " pgrad: ",pginfn, "\n")
 
   #######################
   #  Main iterative loop
@@ -172,11 +187,13 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
   while( pginfn > gtol & iter <= maxit ) {
       iter <- iter + 1
  
-      d <- try(project(par - lambda * g, lower, upper, ...), silent=TRUE)
- 
-      if (class(d) == "try-error" | any(is.nan(d))  ) {
-        lsflag <- 4
-        break
+   d <- par - lambda * g
+   if (prj){
+     d <- try(do.call(project,  append(list(d), projectArgs)), silent=TRUE)
+     if (class(d) == "try-error" | any(is.nan(d))  ) {
+          lsflag <- 4
+          break
+          }
         }
  
       d <- d - par
@@ -224,12 +241,14 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
       par <- pnew
       g   <- gnew
  
-      pg <- try(project(par - g, lower, upper, ...), silent=TRUE)
- 
-      if (class(pg) == "try-error" | any(is.nan(pg)) ) {
-  	lsflag <- 4
-  	break
-  	}
+      pg <- par - g
+      if (prj){
+        pg <- try(do.call(project, append(list(pg), projectArgs)), silent=TRUE)
+        if (class(pg) == "try-error" | any(is.nan(pg)) ) {
+  	  lsflag <- 4
+  	  break
+  	  }
+	}
 
       pg <- pg - par
       pg2n <- sqrt(sum(pg*pg))
@@ -256,6 +275,7 @@ spg <- function(par, fn, gr=NULL, method=3, project=NULL,
     if (pginfn <= gtol) conv <- list(type=0, message="Successful convergence")
     if (iter >= maxit)  conv <- list(type=1, message="Maximum number of iterations exceeded")
     f.rep <- (-1)^maximize * fbest  # This bug was fixed by Ravi Varadhan.  March 29, 2010.
+    par <- pbest
     } else {
       par <- pbest
       f.rep <- f <- (-1)^maximize * fbest
